import hashlib
import hmac

from ..base import AbstractSignature

__title__ = 'ska.signatures.hmac_sha1'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2019 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('HMACSHA1Signature',)


class HMACSHA1Signature(AbstractSignature):
    """HMAC SHA-1 signature."""

    @classmethod
    def make_hash(cls, auth_user, secret_key, valid_until=None, extra=None):
        """Make hash.

        :param str auth_user:
        :param str secret_key:
        :param float|str valid_until: Unix timestamp, valid until.
        :param dict extra: Additional variables to be added.
        :return str:
        """
        if not extra:
            extra = {}

        raw_hmac = hmac.new(
            cls.make_secret_key(secret_key),
            cls.get_base(auth_user, valid_until, extra=extra),
            hashlib.sha1
        ).digest()

        return raw_hmac



__title__ = 'ska.exceptions'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2019 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'BaseSkaException',
    'ImproperlyConfigured',
    'InvalidData',
)


class BaseSkaException(Exception):
    """Base exception."""


class ImproperlyConfigured(BaseSkaException):
    """Improperly configured exception.

    Raised when developer didn't configure/write the code properly.
    """


class InvalidData(BaseSkaException):
    """Invalid data exception.

    Raised when invalid data (tampered) is detected.
    """



"""
Application defaults.

- `SIGNATURE_LIFETIME` (int): Signature lifetime in seconds. Default value is
  600 (seconds).
- `DEFAULT_SIGNATURE_PARAM` (str): Default name of the REQUEST param holding
  the generated signature value. Default value is `signature`.
- `DEFAULT_AUTH_USER_PARAM` (str): Default name of the REQUEST param holding
  the ``auth_user`` value. Default value is `auth_user`.
- `DEFAULT_VALID_UNTIL_PARAM` (str): Default name of the REQUEST param holding
  the ``valid_until`` value. Default value is `valid_until`.
- `DEFAULT_TIME_ZONE_PARAM` (str): Default name of the REQUEST param holding
  the ``time_zone`` value. Default value is `time_zone`.
- `DEFAULT_EXTRA_PARAM` (str): Default name of the REQUEST param holding the
  ``extra`` value. Default value is `extra`.
- `DEFAULT_PROVIDER_PARAM` (str): Default name of the REQUEST param holding
  the ``provider`` value. Default value is `provider`.
- `DEFAULT_URL_SUFFIX` (str): Suffix to add after the ``endpoint_url`` and
  before the appended signature params.
- `DEFAULT_RESERVED_PARAMS` (list): List of GET params reserved by default.
  Users should not be allowed to use them.
"""

__title__ = 'ska.defaults'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2019 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'TIMESTAMP_FORMAT',
    'SIGNATURE_LIFETIME',
    'DEFAULT_SIGNATURE_PARAM',
    'DEFAULT_AUTH_USER_PARAM',
    'DEFAULT_VALID_UNTIL_PARAM',
    'DEFAULT_TIME_ZONE_PARAM',
    'DEFAULT_EXTRA_PARAM',
    'DEFAULT_PROVIDER_PARAM',
    'DEFAULT_URL_SUFFIX',
    'DEFAULT_RESERVED_PARAMS',
    'DEBUG',
)

TIMESTAMP_FORMAT = "%Y-%m-%dT%H:%M:%S"

# Signature lifetime in seconds
SIGNATURE_LIFETIME = 600

# Default name of the REQUEST param holding the generated signature value.
DEFAULT_SIGNATURE_PARAM = 'signature'

# Default name of the REQUEST param holding the ``auth_user`` value.
DEFAULT_AUTH_USER_PARAM = 'auth_user'

# Default name of the REQUEST param holding the ``valid_until`` value.
DEFAULT_VALID_UNTIL_PARAM = 'valid_until'

# Default name of the REQUEST param holding the ``time_zone`` value.
DEFAULT_TIME_ZONE_PARAM = 'time_zone'

# Default name of the REQUEST param holding the ``extra`` value.
DEFAULT_EXTRA_PARAM = 'extra'

# Default name of the REQUEST param holding the ``provider`` value.
DEFAULT_PROVIDER_PARAM = 'provider'

# Suffix to add after the ``endpoint_url`` and before the appended signature
# params.
DEFAULT_URL_SUFFIX = '?'

# A list of REQUEST params reserved by default. Users should not be allowed
# to use them.
DEFAULT_RESERVED_PARAMS = (
    DEFAULT_SIGNATURE_PARAM,
    DEFAULT_AUTH_USER_PARAM,
    DEFAULT_VALID_UNTIL_PARAM,
    DEFAULT_TIME_ZONE_PARAM,
    DEFAULT_EXTRA_PARAM,
    DEFAULT_PROVIDER_PARAM,
)

DEBUG = False



from __future__ import absolute_import

from django import template
from django.core.exceptions import ImproperlyConfigured

from ..... import sign_url as ska_sign_url
from .....defaults import (
    DEFAULT_AUTH_USER_PARAM,
    DEFAULT_EXTRA_PARAM,
    DEFAULT_PROVIDER_PARAM,
    DEFAULT_SIGNATURE_PARAM,
    DEFAULT_URL_SUFFIX,
    DEFAULT_VALID_UNTIL_PARAM,
    SIGNATURE_LIFETIME,
)
from .....signatures import Signature
from ..settings import SECRET_KEY, PROVIDERS

__title__ = 'ska.contrib.django.ska.templatetags.ska_tags'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2019 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'sign_url',
    'provider_sign_url',
)


register = template.Library()


@register.simple_tag(takes_context=True)
def sign_url(context,
             url='',
             auth_user=None,
             secret_key=SECRET_KEY,
             valid_until=None,
             lifetime=SIGNATURE_LIFETIME,
             suffix=DEFAULT_URL_SUFFIX,
             signature_param=DEFAULT_SIGNATURE_PARAM,
             auth_user_param=DEFAULT_AUTH_USER_PARAM,
             valid_until_param=DEFAULT_VALID_UNTIL_PARAM,
             extra=None,
             extra_param=DEFAULT_EXTRA_PARAM,
             signature_cls=Signature):
    """Sign URL."""
    # The `extra` and `extra_param` are not used at the moment.
    if not auth_user:
        auth_user = context['request'].user.get_username()

    return ska_sign_url(
        auth_user=auth_user,
        secret_key=secret_key,
        valid_until=valid_until,
        lifetime=lifetime,
        url=url,
        suffix=suffix,
        signature_param=signature_param,
        auth_user_param=auth_user_param,
        valid_until_param=valid_until_param,
        extra=extra,
        extra_param=extra_param,
        signature_cls=signature_cls
    )


@register.simple_tag(takes_context=True)
def provider_sign_url(context,
                      provider,
                      url='',
                      auth_user=None,
                      valid_until=None,
                      lifetime=SIGNATURE_LIFETIME,
                      suffix=DEFAULT_URL_SUFFIX,
                      signature_param=DEFAULT_SIGNATURE_PARAM,
                      auth_user_param=DEFAULT_AUTH_USER_PARAM,
                      valid_until_param=DEFAULT_VALID_UNTIL_PARAM,
                      extra=None,
                      extra_param=DEFAULT_EXTRA_PARAM,
                      signature_cls=Signature,
                      fail_silently=True):
    """Sign URL."""
    # The `extra` and `extra_param` are not used at the moment.
    if not auth_user:
        auth_user = context['request'].user.get_username()

    if provider not in PROVIDERS:
        if fail_silently:
            return None
        else:
            raise ImproperlyConfigured(
                "Provider {} does not exist".format(provider)
            )
    secret_key = PROVIDERS.get(provider, {}).get('SECRET_KEY', None)

    if extra is None:
        extra = {}

    extra.update({DEFAULT_PROVIDER_PARAM: provider})

    return ska_sign_url(
        auth_user=auth_user,
        secret_key=secret_key,
        valid_until=valid_until,
        lifetime=lifetime,
        url=url,
        suffix=suffix,
        signature_param=signature_param,
        auth_user_param=auth_user_param,
        valid_until_param=valid_until_param,
        extra=extra,
        extra_param=extra_param,
        signature_cls=signature_cls
    )



from __future__ import absolute_import

from django.conf import settings

from ....contrib.django.ska import defaults

__title__ = 'ska.contrib.django.ska.conf'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2019 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('get_setting',)


def get_setting(setting, override=None):
    """Get a setting from `ska.contrib.django.ska` conf module, falling back
    to the default.

    If override is not None, it will be used instead of the setting.
    """
    attr_name = 'SKA_{}'.format(setting)
    if hasattr(settings, attr_name):
        return getattr(settings, attr_name)
    else:
        if hasattr(defaults, setting):
            return getattr(defaults, setting)
        else:
            return override



from ..signatures.hmac_md5 import HMACMD5Signature
from ..signatures.hmac_sha1 import HMACSHA1Signature
from ..signatures.hmac_sha224 import HMACSHA224Signature
from ..signatures.hmac_sha256 import HMACSHA256Signature
from ..signatures.hmac_sha384 import HMACSHA384Signature
from ..signatures.hmac_sha512 import HMACSHA512Signature

__title__ = 'ska.signatures'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2019 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'Signature',
    'HMACMD5Signature',
    'HMACSHA1Signature',
    'HMACSHA224Signature',
    'HMACSHA256Signature',
    'HMACSHA384Signature',
    'HMACSHA512Signature',
)

# Default
Signature = HMACSHA1Signature



from __future__ import absolute_import

import datetime

from ....defaults import DEFAULT_PROVIDER_PARAM
from .models import Signature
from .settings import PROVIDERS, SECRET_KEY

__title__ = 'ska.contrib.django.ska.utils'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2019 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'purge_signature_data',
    'get_secret_key',
    'get_provider_data',
)


def purge_signature_data():
    """Purge old signature data (valid_until < now)."""
    Signature._default_manager \
             .filter(valid_until__lt=datetime.datetime.now()) \
             .delete()


def get_secret_key(data, default=SECRET_KEY):
    """Obtain the secret key from request data given.

    This happens by looking up the secret key by `provider` param from the
    request data in the dictionary of ``PROVIDERS`` defined in settings
    module. If not found, fall back to the ``default`` value given, which is
    by default the globally set secret key.

    :param dict data:
    :param string default: Secret key value to be used as default. By default,
        the globally set secret key is used.
    """
    provider = data.get(DEFAULT_PROVIDER_PARAM, None)
    if provider:
        provider_data = PROVIDERS.get(provider, None)
        if provider_data:
            return provider_data.get('SECRET_KEY', default)

    return default


def get_provider_data(data, settings=None):
    """Obtain the secret key from request data given.

    This happens by looking up the secret key by `provider` param from the
    request data in the dictionary of ``PROVIDERS`` defined in settings
    module. If not found, fall back to the ``default`` value given, which is
    by default the globally set secret key.

    :param dict data:
    :param dict settings: Settings dict.
    """
    if not settings or not isinstance(settings, dict):
        settings = PROVIDERS
    provider = data.get(DEFAULT_PROVIDER_PARAM, None)
    if provider:
        return settings.get(provider, None)
    return {}



from __future__ import absolute_import

from nine import versions

from ..views import constance_login

if versions.DJANGO_GTE_2_1:
    from django.urls import re_path as url
else:
    from django.conf.urls import url


__title__ = 'ska.contrib.django.ska.urls.constance_urls'
__author__ = 'Artur Barseghyan'
__copyright__ = '2013-2019 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('urlpatterns',)


urlpatterns = [
    url(r'^login/', constance_login, name="ska.login"),
]



from __future__ import absolute_import, print_function

import datetime
import logging
import time
import unittest

from django.core import mail
from django.core.management import call_command
from django.test import Client, TransactionTestCase, override_settings

import mock

import pytest

from ska import sign_url
from ska.contrib.django.ska.models import Signature
from ska.contrib.django.ska import settings as ska_settings
from ska.defaults import DEFAULT_PROVIDER_PARAM

import factories

from .helpers import log_info

__title__ = 'ska.contrib.django.ska.tests.test_default_authentication_backend'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2019 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'SkaAuthenticationBackendTest',
)

logger = logging.getLogger(__name__)


OVERRIDE_SETTINGS_DB_STORE_SIGNATURES_KWARGS = {
    'SKA_DB_STORE_SIGNATURES': True,
    'SKA_DB_PERFORM_SIGNATURE_CHECK': True,
}

# *********************************************************************
# *********************************************************************
# *********************************************************************

SKA_TEST_USER_USERNAME = factories.TEST_ADMIN_USERNAME
SKA_TEST_USER_PASSWORD = factories.TEST_PASSWORD


@pytest.mark.django_db
class SkaAuthenticationBackendTest(TransactionTestCase):
    """Tests for auth backend."""

    pytestmark = pytest.mark.django_db

    def setUp(self):
        self.AUTH_USER = 'test_auth_backend_user'
        self.AUTH_USER_EMAIL = 'test_ska_auth_user@mail.example.com'
        self.AUTH_USER_FIRST_NAME = 'John'
        self.AUTH_USER_LAST_NAME = 'Doe'
        self.PROVIDER_NAME = 'client_1.admins'
        self.LOGIN_URL = '/ska/login/'

    def __test_login(self,
                     secret_key,
                     response_codes,
                     provider_name=None,
                     logout=False,
                     test_email_data=None,
                     do_signature_check=True,
                     auth_user=None,
                     auth_user_email=None,
                     debug_info=""):
        """Test login.

        :param secret_key:
        :param response_codes: First response code is used
        :param provider_name:
        :param logout:
        :param test_email_data:
        :param do_signature_check: It's a tuple. The second code is
            used only if both `DB_STORE_SIGNATURES` and
            `DB_PERFORM_SIGNATURE_CHECK` values are True, which means
            that users can't use same signatures twice.
        :param auth_user:
        :param auth_user_email:
        :return:
        """
        flow = []

        if not auth_user:
            auth_user = self.AUTH_USER
        if not auth_user_email:
            auth_user_email = self.AUTH_USER_EMAIL

        first_response_code, second_response_code = response_codes

        # Testing signed URLs
        extra = {
            'email': auth_user_email,
            'first_name': self.AUTH_USER_FIRST_NAME,
            'last_name': self.AUTH_USER_LAST_NAME,
        }
        if provider_name:
            extra.update({DEFAULT_PROVIDER_PARAM: provider_name})

        signed_login_url = sign_url(
            auth_user=auth_user,
            secret_key=secret_key,
            url=self.LOGIN_URL,
            extra=extra
        )

        self.assertIsNotNone(signed_login_url)
        flow.append(('Signed login URL', signed_login_url))

        # Testing view with signed URL
        self._client = Client()
        response = self._client.get(signed_login_url, {})
        response_status_code = getattr(response, 'status_code', None)

        self.assertIn(response_status_code, (first_response_code,))
        flow.append(
            (
                'Response status code for signed URL',
                response_status_code
            )
        )

        if test_email_data:
            self.assertEqual(
                len(mail.outbox),
                len(test_email_data['emails'])
            )

            for _index, _subject in test_email_data['emails'].items():
                self.assertEqual(
                    mail.outbox[_index].subject,
                    _subject
                )

        if logout:
            self.__logout()

        # If both `DB_STORE_SIGNATURES` and `DB_PERFORM_SIGNATURE_CHECK`
        # are set to True, second login attempt shall not be successful.
        if ska_settings.DB_STORE_SIGNATURES \
                and ska_settings.DB_PERFORM_SIGNATURE_CHECK \
                and do_signature_check:
            # Testing again with signed URL and this time, it should fail

            self._client = Client()
            response = self._client.get(signed_login_url, {})
            response_status_code = getattr(response, 'status_code', None)
            self.assertIn(response_status_code, (second_response_code,))
            flow.append(
                (
                    'Response status '
                    'code for signed URL', response_status_code
                )
            )
            if logout:
                self.__logout()

        return flow

    def __logout(self):
        """Log out."""
        self._client.logout()

    @log_info
    def test_01_login(self):
        """Test auth using general ``SECRET_KEY``."""
        return self.__test_login(
            ska_settings.SECRET_KEY,
            [302, 403],
            logout=True,
            debug_info="test_01_login"
        )

    @log_info
    def test_02_provider_login(self):
        """Test auth using ``SECRET_KEY`` defined in ``PROVIDERS``."""
        secret_key = ska_settings.PROVIDERS[self.PROVIDER_NAME]['SECRET_KEY']

        # Authenticate for the first time. There shall be 2 emails
        # for `create` and `info` callbacks.
        self.__test_login(
            secret_key,
            [302, 403],
            self.PROVIDER_NAME,
            logout=True,
            test_email_data={
                'emails': {
                    0: 'Welcome validate::admins!',
                    1: 'Welcome create::admins!',
                    2: 'Welcome info::admins!',
                }
            },
            debug_info="test_02_provider_login::first time"
        )

        # Sleep for just one second between first and the second tests.
        time.sleep(1)

        # Authenticate for the second time. There shall be still 2 emails
        # from the first time login (since we're in the same test method
        # still) and 2 new ones for `get` and `info` callbacks.
        self.__test_login(
            secret_key,
            [302, 403],
            self.PROVIDER_NAME,
            logout=True,
            test_email_data={
                'emails': {
                    # First round
                    0: 'Welcome validate::admins!',
                    1: 'Welcome create::admins!',
                    2: 'Welcome info::admins!',
                    # Second round
                    # TODO: this is weird. validate::admin shall not
                    # appear here twice.
                    3: 'Welcome validate::admins!',
                    4: 'Welcome validate::admins!',
                    5: 'Welcome get::admins!',
                    6: 'Welcome info::admins!',
                }
            },
            do_signature_check=False,
            debug_info="test_02_provider_login::second time"
        )

    @log_info
    def test_03_login_fail_wrong_secret_key(self):
        """Fail test auth using general ``SECRET_KEY``.

        Fail test auth using general ``SECRET_KEY`` providing wrong
        secret key.
        """
        return self.__test_login(
            ska_settings.SECRET_KEY + 'wrong',
            [403, 403],
            logout=True,
            debug_info="test_03_login_fail_wrong_secret_key"
        )

    @log_info
    def test_04_provider_login_fail_wrong_secret_key(self):
        """Fail test authentication.

        Fail test authentication using ``SECRET_KEY`` defined in `
        `PROVIDERS``, providing wrong secret key.
        """
        secret_key = ska_settings.PROVIDERS[self.PROVIDER_NAME]['SECRET_KEY']
        return self.__test_login(
            secret_key + 'wrong',
            [403, 403],
            self.PROVIDER_NAME,
            logout=True
        )

    @log_info
    def test_05_provider_login_fail_wrong_provider(self):
        """Test provider login fail wrong provider.

        Fail test authentication using ``SECRET_KEY`` defined in
        ``PROVIDERS``, providing wrong provider name.
        """
        secret_key = ska_settings.PROVIDERS[self.PROVIDER_NAME]['SECRET_KEY']
        return self.__test_login(
            secret_key + 'wrong',
            [403, 403],
            self.PROVIDER_NAME + 'wrong',
            logout=True,
            debug_info="test_05_provider_login_fail_wrong_provider"
        )

    @log_info
    @override_settings(**OVERRIDE_SETTINGS_DB_STORE_SIGNATURES_KWARGS)
    @mock.patch(
        'ska.contrib.django.ska.settings.DB_STORE_SIGNATURES',
        True
    )
    @mock.patch(
        'ska.contrib.django.ska.settings.DB_PERFORM_SIGNATURE_CHECK',
        True
    )
    def test_06_purge_stored_signatures_data(self):
        """Test purge stored signature data."""
        secret_key = ska_settings.PROVIDERS[self.PROVIDER_NAME]['SECRET_KEY']
        # Login
        self.__test_login(
            secret_key,
            [302, 403],
            self.PROVIDER_NAME,
            logout=True,
            debug_info="test_06_purge_stored_signatures_data"
        )

        # Duplicate signatures
        signature = Signature.objects.first()
        signature.id = None
        signature.signature += '0'
        signature.save()
        signature.id = None
        signature.signature += '0'
        signature.save()
        # There should be 3
        self.assertEqual(Signature.objects.all().count(), 3)

        # Manually set valid_until to no longer valid so that we can
        # test
        invalid_until = (
            datetime.datetime.now() - datetime.timedelta(minutes=20)
        )
        Signature.objects.update(
            valid_until=invalid_until
        )

        # Call purge command
        call_command('ska_purge_stored_signature_data')
        # All old records shall be gone
        self.assertEqual(Signature.objects.all().count(), 0)

    @log_info
    def test_07_provider_login_forbidden_email(self):
        """Test auth using ``SECRET_KEY`` defined in ``PROVIDERS``."""
        secret_key = ska_settings.PROVIDERS[self.PROVIDER_NAME]['SECRET_KEY']

        # Authenticate for the first time. There shall be 2 emails
        # for `create` and `info` callbacks.
        self.__test_login(
            secret_key,
            [403, 403],
            self.PROVIDER_NAME,
            logout=True,
            test_email_data={
                'emails': {}
            },
            auth_user_email='forbidden@example.com',
            debug_info="test_07_provider_login_forbidden_email"
        )

    @log_info
    def test_08_provider_login_forbidden_username(self):
        """Test auth using ``SECRET_KEY`` defined in ``PROVIDERS``."""
        secret_key = ska_settings.PROVIDERS[self.PROVIDER_NAME]['SECRET_KEY']

        # Authenticate for the first time. There shall be 2 emails
        # for `create` and `info` callbacks.
        self.__test_login(
            secret_key,
            [403, 403],
            self.PROVIDER_NAME,
            logout=True,
            test_email_data={
                'emails': {}
            },
            auth_user='forbidden_username',
            debug_info="test_08_provider_login_forbidden_username"
        )


if __name__ == "__main__":
    # Tests
    unittest.main()



from __future__ import absolute_import

import logging

from .base import BaseSkaAuthenticationBackend

logger = logging.getLogger(__file__)

__title__ = 'ska.contrib.django.ska.backends.default_backend'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2019 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('SkaAuthenticationBackend',)


class SkaAuthenticationBackend(BaseSkaAuthenticationBackend):
    """Authentication backend."""

    def get_settings(self):
        """Get settings.

        :return:
        """
        return {}

    def get_secret_key(self):
        """Get secret key.

        :return:
        """



import datetime
import logging

from six import PY3

from .. import TIMESTAMP_FORMAT

try:
    from six.moves.urllib.parse import urlparse, parse_qs
except ImportError as err:
    if PY3:
        from urllib.parse import urlparse, parse_qs
    else:
        from urlparse import urlparse, parse_qs

__title__ = 'ska.tests.base'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2019 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'LOG_INFO',
    'log_info',
    'timestamp_to_human_readable',
    'parse_url_params',
)


logger = logging.getLogger(__name__)
LOG_INFO = True


def log_info(func):
    """Prints some useful info."""
    if not LOG_INFO:
        return func

    def inner(self, *args, **kwargs):
        """Inner"""
        result = func(self, *args, **kwargs)

        logger.debug('\n\n%s', func.__name__)
        logger.debug('============================')
        if func.__doc__:
            logger.debug('""" %s """', func.__doc__.strip())
        logger.debug('----------------------------')
        if result is not None:
            logger.debug(result)
        logger.debug('\n++++++++++++++++++++++++++++')

        return result
    return inner


def timestamp_to_human_readable(timestamp):
    """Convert Unix timestamp to human readable string.

    :param float:
    :return str:
    """
    val = datetime.datetime.fromtimestamp(float(timestamp))
    return val.strftime(TIMESTAMP_FORMAT)


def parse_url_params(url):
    """Parses URL params.

    :param str url:
    :return dict:
    """
    data = parse_qs(urlparse(url).query)
    for key, val in data.items():
        data[key] = val[0]

    return data



from ....settings import (
    SECRET_KEY,
    PROVIDERS,
)

from .base import (
    BaseProviderSignedRequestRequired,
    BaseSignedRequestRequired,
)

__title__ = 'ska.contrib.django.ska.integration.drf.permissions.' \
            'default_permissions'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2019 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'SignedRequestRequired',
    'ProviderSignedRequestRequired',
)


class SignedRequestRequired(BaseSignedRequestRequired):
    """Signed request required permission."""

    def get_settings(self):
        return {
            'SECRET_KEY': SECRET_KEY,
        }


class ProviderSignedRequestRequired(BaseProviderSignedRequestRequired):
    """Provider signed request required permission."""

    def get_settings(self):
        return PROVIDERS



from __future__ import print_function

import argparse

from .shortcuts import sign_url

__title__ = 'ska.generate_signed_url'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2019 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('main',)


def main():
    """Prints signed URL to console.

    :example:

        python src/ska/generate_signature.py -au user -sk test
    """
    parser = argparse.ArgumentParser(description="""
    Generates signed URL.
    """)

    parser.add_argument("-au",
                        "--auth-user",
                        dest="auth_user",
                        type=str,
                        help="`auth_user` value",
                        metavar="AUTH_USER")
    parser.add_argument("-sk",
                        "--secret-key",
                        dest="secret_key",
                        type=str,
                        help="`secret_key` value",
                        metavar="SECRET_KEY")
    parser.add_argument("-vu",
                        "--valid-until",
                        dest="valid_until",
                        type=float,
                        help="`valid_until` value",
                        metavar="VALID_UNTIL")
    parser.add_argument("-l",
                        "--lifetime",
                        dest="lifetime",
                        type=int,
                        help="`lifetime` value",
                        metavar="LIFETIME")
    parser.add_argument("-u",
                        "--url",
                        dest="url",
                        type=str,
                        help="URL to sign",
                        metavar="URL")
    parser.add_argument("-s",
                        "--suffix",
                        dest="suffix",
                        type=str,
                        help="Added after the `url`.",
                        metavar="SUFFIX")
    parser.add_argument("-sp",
                        "--signature-param",
                        dest="signature_param",
                        type=str,
                        help="GET param holding the `signature` value",
                        metavar="SIGNATURE_PARAM")
    parser.add_argument("-aup",
                        "--auth-user-param",
                        dest="auth_user_param",
                        type=str,
                        help="GET param holding the `auth_user` value",
                        metavar="AUTH_USER_PARAM")
    parser.add_argument("-vup",
                        "--valid-until-param",
                        dest="valid_until_param",
                        type=str,
                        help="GET param holding the `auth_user` value",
                        metavar="VALID_UNTIL_PARAM")

    args = parser.parse_args()

    kwargs = {}

    if args.auth_user:
        kwargs.update({'auth_user': args.auth_user})

    if args.secret_key:
        kwargs.update({'secret_key': args.secret_key})

    if args.valid_until:
        kwargs.update({'valid_until': args.valid_until})

    if args.lifetime:
        kwargs.update({'lifetime': args.lifetime})

    if args.url:
        kwargs.update({'url': args.url})

    if args.suffix:
        kwargs.update({'suffix': args.suffix})

    if args.signature_param:
        kwargs.update({'signature_param': args.signature_param})

    if args.auth_user_param:
        kwargs.update({'auth_user_param': args.auth_user_param})

    if args.valid_until_param:
        kwargs.update({'valid_until_param': args.valid_until_param})

    try:
        signed_url = sign_url(**kwargs)
        print(signed_url)
    except Exception as err:
        print(err)


if __name__ == "__main__":
    main()



from __future__ import absolute_import

from constance import config

from django.contrib import messages
from django.contrib.auth import authenticate, login as auth_login
from django.http import HttpResponseForbidden, HttpResponseRedirect
from django.utils.translation import ugettext, ugettext_lazy as _

from nine import versions

from ..settings import REDIRECT_AFTER_LOGIN
from ..utils import get_provider_data

__title__ = 'ska.contrib.django.ska.views.constance_views'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2019 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('constance_login',)


def constance_login(request):
    """Login.

    Authenticate with `ska` token into Django.

    :param django.http.HttpRequest request:
    :return django.http.HttpResponse:
    """
    user = authenticate(request=request)
    next_url = request.GET.get('next', None)

    if not next_url:
        if versions.DJANGO_GTE_1_7:
            request_data = request.GET.dict()
        else:
            request_data = request.REQUEST
        settings = config.SKA_PROVIDERS
        provider_data = get_provider_data(request_data, settings)
        if provider_data:
            next_url = provider_data.get(
                'REDIRECT_AFTER_LOGIN',
                REDIRECT_AFTER_LOGIN
            )

    if not next_url:
        next_url = '/'

    if user is not None:
        auth_login(request, user)
        name = user.first_name or user.username
        messages.info(
            request, ugettext("Login succeeded. Welcome, {0}.").format(name)
        )
        return HttpResponseRedirect(next_url)
    else:
        return HttpResponseForbidden(_("Authentication error!"))



from __future__ import absolute_import
import json
import logging

from constance import config

from django.conf import settings

from .base import BaseSkaAuthenticationBackend

logger = logging.getLogger(__file__)

__title__ = 'ska.contrib.django.ska.backends.constance_backend'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2019 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('SkaAuthenticationConstanceBackend',)


class SkaAuthenticationConstanceBackend(BaseSkaAuthenticationBackend):
    """Authentication backend."""

    def get_settings(self):
        """Get settings.

        :return:
        """
        parse_from_json = getattr(
            settings,
            'SKA_CONSTANCE_SETTINGS_PARSE_FROM_JSON',
            False
        )
        if parse_from_json:
            try:
                _settings = json.loads(config.SKA_PROVIDERS)
                return _settings
            except ValueError as err:
                pass
        return config.SKA_PROVIDERS

    def get_secret_key(self):
        """Get secret key.

        :return:
        """
        return config.SKA_SECRET_KEY



from .base import *
from .default_permissions import *
from .constance_permissions import *

__title__ = 'ska.contrib.django.ska.integration.drf.permissions'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2019 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'AbstractSignedRequestRequired',
    'BaseSignedRequestRequired',
    'BaseProviderSignedRequestRequired',
    'SignedRequestRequired',
    'ProviderSignedRequestRequired',
    'ConstanceSignedRequestRequired',
    'ConstanceProviderSignedRequestRequired',
)



import unittest

from bs4 import BeautifulSoup

from django.test import Client, TransactionTestCase, override_settings

from nine import versions

from .helpers import log_info

import factories

if versions.DJANGO_GTE_1_10:
    from django.urls import reverse
else:
    from django.core.urlresolvers import reverse

__title__ = 'ska.contrib.django.ska.tests.test_templatetags'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2019 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'SkaTagsTest',
    'SkaConstanceTagsTest',
)


OVERRIDE_SETTINGS_KWARGS = {
    'AUTHENTICATION_BACKENDS': (
        'ska.contrib.django.ska.backends.constance_backend.'
        'SkaAuthenticationConstanceBackend',
        'django.contrib.auth.backends.ModelBackend',
    ),
    'ROOT_URLCONF': 'constance_urls',
}


class BaseSkaTagsTest(TransactionTestCase):
    """Base template tags tests."""

    @classmethod
    def setUpClass(cls):
        cls.templatetags_url = reverse('foo.templatetags')

    def setUp(self):
        """Set up."""
        factories.SkaSecretKeyConstanceFactory()
        factories.SkaProvidersConstanceFactory()

    def _test_sign_url(self, sign_in_link_class):
        """Test `sign_url` or `provider_sign_url` template tag."""
        client = Client()

        response_1 = client.get(self.templatetags_url)
        soup_1 = BeautifulSoup(
            getattr(response_1, 'content', ''),
            features="html.parser"
        )

        sign_in_link_element_1 = soup_1.find(
            'a',
            attrs={'class': sign_in_link_class}
        )
        sign_in_url = sign_in_link_element_1.attrs.get('href')
        sign_in_element_classes_1 = sign_in_link_element_1.attrs.get('class')
        self.assertIn('logged-out', sign_in_element_classes_1)
        self.assertNotIn('logged-in', sign_in_element_classes_1)

        response_2 = client.get(sign_in_url, follow=True)
        soup_2 = BeautifulSoup(
            getattr(response_2, 'content', ''),
            features="html.parser"
        )
        sign_in_link_element_2 = soup_2.find(
            'a',
            attrs={'class': sign_in_link_class}
        )
        sign_in_element_classes_2 = sign_in_link_element_2.attrs.get('class')
        self.assertNotIn('logged-out', sign_in_element_classes_2)
        self.assertIn('logged-in', sign_in_element_classes_2)


class SkaTagsTest(BaseSkaTagsTest):
    """Testing `ska_tags` functionality."""

    @log_info
    def test_01_sign_url(self):
        """Test `sign_url` template tag."""
        return self._test_sign_url(
            sign_in_link_class='signed-url'
        )

    @log_info
    def test_02_provider_sign_url(self):
        """Test `provider_sign_url` template tag."""
        return self._test_sign_url(
            sign_in_link_class='provider-signed-url'
        )


class SkaConstanceTagsTest(BaseSkaTagsTest):
    """Testing `ska_constance_tags` functionality."""

    @log_info
    @override_settings(**OVERRIDE_SETTINGS_KWARGS)
    def test_01_sign_url(self):
        """Test `sign_url` template tag."""
        return self._test_sign_url(
            sign_in_link_class='signed-url'
        )

    @log_info
    @override_settings(**OVERRIDE_SETTINGS_KWARGS)
    def test_02_provider_sign_url(self):
        """Test `provider_sign_url` template tag."""
        return self._test_sign_url(
            sign_in_link_class='provider-signed-url'
        )


if __name__ == '__main__':
    unittest.main()



"""
Testing Django REST Framework JWT token view for ska.
"""

import logging
import unittest

from constance import config
from django.test import TransactionTestCase, override_settings
import pytest
from rest_framework import status
from rest_framework.test import APIClient

from nine import versions

from ska import sign_url
from ska.contrib.django.ska.settings import SECRET_KEY, PROVIDERS
from ska.defaults import DEFAULT_PROVIDER_PARAM

import factories

if versions.DJANGO_GTE_1_10:
    from django.urls import reverse
else:
    from django.core.urlresolvers import reverse

__title__ = 'ska.contrib.django.ska.tests.test_drf_integration_view_jwt_token'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2019 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'DRFIntegrationViewJwtTokenTestCase',
    'DRFIntegrationViewJwtTokenConstanceTestCase',
)

LOGGER = logging.getLogger(__name__)

OVERRIDE_SETTINGS_KWARGS = {
    'AUTHENTICATION_BACKENDS': (
        'ska.contrib.django.ska.backends.constance_backend.'
        'SkaAuthenticationConstanceBackend',
        'django.contrib.auth.backends.ModelBackend',
    ),
    'ROOT_URLCONF': 'constance_urls',
}


@pytest.mark.django_db
class BaseDRFIntegrationViewJwtTokenTestCase(TransactionTestCase):
    """Django REST framework integration view jwt token test case."""

    pytestmark = pytest.mark.django_db

    @classmethod
    def setUpClass(cls):
        """Set up class."""
        cls.client = APIClient()

        cls.drf_obtain_jwt_token_url = reverse('ska.obtain_jwt_token')

        cls.AUTH_USER = 'test_auth_backend_user'
        cls.AUTH_USER_EMAIL = 'test_ska_auth_user@mail.example.com'
        cls.AUTH_USER_FIRST_NAME = 'John'
        cls.AUTH_USER_LAST_NAME = 'Doe'
        cls.PROVIDER_NAME = 'client_1.admins'

    def setUp(self):
        """Set up."""
        factories.SkaSecretKeyConstanceFactory()
        factories.SkaProvidersConstanceFactory()

    def _test_obtain_jwt_token_request_not_signed_fail(self, url):
        """Fail test obtain JWT token request not signed.

        :return:
        """
        data = {}
        response = self.client.get(url, data)
        self.assertIn(
            response.status_code,
            (
                status.HTTP_401_UNAUTHORIZED,
                # status.HTTP_403_FORBIDDEN,
            )
        )

    def _test_obtain_jwt_token_request_signed(self,
                                              secret_key,
                                              expected_response_code,
                                              url,
                                              auth_user=None,
                                              auth_user_email=None,
                                              provider_name=None,
                                              check_token=True,
                                              debug_info=""):
        """Test obtain JWT token signed requests.

        :return:
        """
        if not auth_user:
            auth_user = self.AUTH_USER
        if not auth_user_email:
            auth_user_email = self.AUTH_USER_EMAIL

        # Testing signed URLs
        extra = {
            'email': auth_user_email,
            'first_name': self.AUTH_USER_FIRST_NAME,
            'last_name': self.AUTH_USER_LAST_NAME,
        }

        if provider_name:
            extra.update({DEFAULT_PROVIDER_PARAM: provider_name})

        signed_url = sign_url(
            auth_user=auth_user,
            secret_key=secret_key,
            url=url,
            extra=extra
        )

        # if debug_info:
        #     signed_url = '{}&debug_info={}'.format(signed_url, debug_info)

        data = {}

        if not isinstance(expected_response_code, (tuple, list)):
            expected_response_code = [expected_response_code]

        response = self.client.get(signed_url, data)

        self.assertIn(response.status_code, expected_response_code)
        if check_token:
            self.assertIn('token', response.data)


@pytest.mark.django_db
class DRFIntegrationViewJwtTokenTestCase(
    BaseDRFIntegrationViewJwtTokenTestCase
):
    """Django REST framework integration view JWT token test case."""

    pytestmark = pytest.mark.django_db

    # **************************************************************
    # ********************* Default permissions ********************
    # **************************************************************

    def test_obtain_jwt_token_request_not_signed_fail(self):
        """Fail test permissions provider list request not signed.

        :return:
        """
        self._test_obtain_jwt_token_request_not_signed_fail(
            self.drf_obtain_jwt_token_url
        )

    def test_obtain_jwt_token_provider_request_signed(self):
        """Test provider obtain JWT token signed request.

        :return:
        """
        secret_key = PROVIDERS[self.PROVIDER_NAME]['SECRET_KEY']
        self._test_obtain_jwt_token_request_signed(
            secret_key,
            status.HTTP_200_OK,
            self.drf_obtain_jwt_token_url,
            auth_user=self.AUTH_USER,
            auth_user_email=self.AUTH_USER_EMAIL,
            provider_name=self.PROVIDER_NAME
        )

    def test_obtain_jwt_token_provider_request_signed_wrong_secret_key_fail(
        self
    ):
        """Test provider obtain JWT token signed request wrong secret key.

        :return:
        """
        secret_key = PROVIDERS[self.PROVIDER_NAME]['SECRET_KEY']
        self._test_obtain_jwt_token_request_signed(
            "{}w".format(secret_key),
            (
                status.HTTP_401_UNAUTHORIZED,
                status.HTTP_403_FORBIDDEN,
            ),
            self.drf_obtain_jwt_token_url,
            auth_user=self.AUTH_USER,
            auth_user_email=self.AUTH_USER_EMAIL,
            provider_name=self.PROVIDER_NAME,
            check_token=False
        )

    def test_obtain_jwt_token_request_signed(self):
        """Test obtain JWT token signed request.

        :return:
        """
        secret_key = SECRET_KEY
        self._test_obtain_jwt_token_request_signed(
            secret_key,
            status.HTTP_200_OK,
            self.drf_obtain_jwt_token_url,
            auth_user=self.AUTH_USER,
            auth_user_email=self.AUTH_USER_EMAIL
        )

    def test_obtain_jwt_token_request_signed_wrong_secret_key_fail(self):
        """Test obtain JWT token signed request wrong secret key.

        :return:
        """
        secret_key = SECRET_KEY
        self._test_obtain_jwt_token_request_signed(
            "{}w".format(secret_key),
            (
                status.HTTP_401_UNAUTHORIZED,
                status.HTTP_403_FORBIDDEN,
            ),
            self.drf_obtain_jwt_token_url,
            auth_user=self.AUTH_USER,
            auth_user_email=self.AUTH_USER_EMAIL,
            check_token=False
        )


@pytest.mark.django_db
class DRFIntegrationViewJwtTokenConstanceTestCase(
    BaseDRFIntegrationViewJwtTokenTestCase
):
    """Django REST framework integration view JWT token constance test case."""

    pytestmark = pytest.mark.django_db

    # **************************************************************
    # ********************* Default permissions ********************
    # **************************************************************

    @override_settings(**OVERRIDE_SETTINGS_KWARGS)
    def test_obtain_jwt_token_request_not_signed_fail(self):
        """Fail test permissions provider list request not signed.

        :return:
        """
        self._test_obtain_jwt_token_request_not_signed_fail(
            self.drf_obtain_jwt_token_url
        )

    @override_settings(**OVERRIDE_SETTINGS_KWARGS)
    def test_obtain_jwt_token_provider_request_signed(self):
        """Test provider obtain JWT token signed request.

        :return:
        """
        secret_key = config.SKA_PROVIDERS[self.PROVIDER_NAME]['SECRET_KEY']
        self._test_obtain_jwt_token_request_signed(
            secret_key,
            status.HTTP_200_OK,
            self.drf_obtain_jwt_token_url,
            auth_user=self.AUTH_USER,
            auth_user_email=self.AUTH_USER_EMAIL,
            provider_name=self.PROVIDER_NAME
        )

    @override_settings(**OVERRIDE_SETTINGS_KWARGS)
    def test_obtain_jwt_token_provider_request_signed_wrong_secret_key_fail(
        self
    ):
        """Test provider obtain JWT token signed request wrong secret key.

        :return:
        """
        secret_key = config.SKA_PROVIDERS[self.PROVIDER_NAME]['SECRET_KEY']
        self._test_obtain_jwt_token_request_signed(
            "{}w".format(secret_key),
            (
                status.HTTP_401_UNAUTHORIZED,
                status.HTTP_403_FORBIDDEN,
            ),
            self.drf_obtain_jwt_token_url,
            auth_user=self.AUTH_USER,
            auth_user_email=self.AUTH_USER_EMAIL,
            provider_name=self.PROVIDER_NAME,
            check_token=False
        )

    @override_settings(**OVERRIDE_SETTINGS_KWARGS)
    def test_obtain_jwt_token_request_signed(self):
        """Test obtain JWT token signed request.

        :return:
        """
        secret_key = config.SKA_SECRET_KEY
        self._test_obtain_jwt_token_request_signed(
            secret_key,
            status.HTTP_200_OK,
            self.drf_obtain_jwt_token_url,
            auth_user=self.AUTH_USER,
            auth_user_email=self.AUTH_USER_EMAIL
        )

    @override_settings(**OVERRIDE_SETTINGS_KWARGS)
    def test_obtain_jwt_token_request_signed_wrong_secret_key_fail(self):
        """Test obtain JWT token signed request wrong secret key.

        :return:
        """
        secret_key = config.SKA_SECRET_KEY
        self._test_obtain_jwt_token_request_signed(
            "{}w".format(secret_key),
            (
                status.HTTP_401_UNAUTHORIZED,
                status.HTTP_403_FORBIDDEN,
            ),
            self.drf_obtain_jwt_token_url,
            auth_user=self.AUTH_USER,
            auth_user_email=self.AUTH_USER_EMAIL,
            check_token=False
        )


if __name__ == "__main__":
    # Tests
    unittest.main()



from __future__ import absolute_import

import logging

from django.db import models
from django.utils.translation import ugettext_lazy as _

from six import python_2_unicode_compatible

logger = logging.getLogger(__name__)

__title__ = 'ska.contrib.django.ska.models'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2019 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('Signature',)


@python_2_unicode_compatible
class Signature(models.Model):
    """Signature.

    :Properties:
        - `signature` (str): Signature generated.
        - `auth_user` (str): Auth user.
        - `valid_until` (datetime.datetime): Valid until.
        - `created` (datetime.datetime): Time added.
    """
    signature = models.CharField(_("Signature"), max_length=255)
    auth_user = models.CharField(_("Auth user"), max_length=255)
    valid_until = models.DateTimeField(_("Valid until"))
    created = models.DateTimeField(_("Date created"), auto_now_add=True)

    class Meta(object):
        """Meta class."""

        verbose_name = _("Token")
        verbose_name_plural = _("Tokens")
        unique_together = (('signature', 'auth_user', 'valid_until'),)

    def __str__(self):
        return "{0}{1}{2}".format(self.signature, self.auth_user,
                                  self.valid_until)



from .base import *
from .constance_backend import *
from .default_backends import *

__title__ = 'ska.contrib.django.ska.backends'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2019 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'BaseSkaAuthenticationBackend',
    'SkaAuthenticationBackend',
    'SkaAuthenticationConstanceBackend',
)



from __future__ import absolute_import
from django.apps import AppConfig

__title__ = 'ska.contrib.django.ska.apps'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2019 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('Config',)


class Config(AppConfig):
    """Config."""

    name = 'ska.contrib.django.ska'
    # label = 'ska_contrib_django_ska'
    label = 'ska'



from django.contrib.auth import authenticate

from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework.exceptions import AuthenticationFailed
from rest_framework_jwt.settings import api_settings

jwt_payload_handler = api_settings.JWT_PAYLOAD_HANDLER
jwt_encode_handler = api_settings.JWT_ENCODE_HANDLER
jwt_decode_handler = api_settings.JWT_DECODE_HANDLER
jwt_get_username_from_payload = api_settings.JWT_PAYLOAD_GET_USERNAME_HANDLER

__title__ = 'ska.contrib.django.ska.integration.drf.views.jwt_token'
__author__ = 'Artur Barseghyan'
__copyright__ = '2013-2019 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'ObtainJSONWebTokenView',
)


class ObtainJSONWebTokenView(APIView):
    """Obtain a JSON web token."""

    def get(self, request, format=None):
        user = authenticate(request=request)

        if user is not None:
            if user.is_active:
                payload = jwt_payload_handler(user)
                data = {'token': jwt_encode_handler(payload)}
                return Response(data)
            else:
                raise AuthenticationFailed("User is not active")

        raise AuthenticationFailed("Authentication failed")



from __future__ import absolute_import

from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from .models import Signature

__title__ = 'ska.contrib.django.ska.admin'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2019 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('SignatureAdmin',)


class SignatureAdmin(admin.ModelAdmin):
    """Signature admin."""

    list_display = ('signature', 'auth_user', 'valid_until', 'created',)
    readonly_fields = ('created',)
    list_filter = ('auth_user',)
    fieldsets = (
        (None, {
            'fields': ('signature', 'auth_user', 'valid_until')
        }),
        (_('Additional'), {
            'classes': ('collapse',),
            'fields': ('created',)
        }),
    )

    class Meta(object):
        """Meta class."""

        app_label = _('Signature')


admin.site.register(Signature, SignatureAdmin)



"""
Testing Django REST Framework permissions for ska.
"""

import logging
import unittest

from constance import config
from django.test import TransactionTestCase, override_settings
import pytest
from rest_framework import status
from rest_framework.test import APIClient

from nine import versions

from ska import sign_url
from ska.contrib.django.ska.settings import SECRET_KEY, PROVIDERS
from ska.defaults import DEFAULT_PROVIDER_PARAM

import factories

if versions.DJANGO_GTE_1_10:
    from django.urls import reverse
else:
    from django.core.urlresolvers import reverse

__title__ = 'ska.contrib.django.ska.tests.test_drf_integration_permissions'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2019 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'DRFIntegrationPermissionsTestCase',
    'DRFIntegrationPermissionsConstanceTestCase',
)

LOGGER = logging.getLogger(__name__)

OVERRIDE_SETTINGS_KWARGS = {
    'AUTHENTICATION_BACKENDS': (
        'ska.contrib.django.ska.backends.constance_backend.'
        'SkaAuthenticationConstanceBackend',
        'django.contrib.auth.backends.ModelBackend',
    ),
    'ROOT_URLCONF': 'constance_urls',
}


@pytest.mark.django_db
class BaseDRFIntegrationPermissionsTestCase(TransactionTestCase):
    """Django REST framework integration permissions test case."""

    pytestmark = pytest.mark.django_db

    @classmethod
    def setUpClass(cls):
        """Set up class."""

        cls.client = APIClient()

        cls.provider_list_url = reverse(
            'fooitemmodel_provider_signed_request_required-list'
        )
        cls.list_url = reverse(
            'fooitemmodel_signed_request_required-list'
        )

        cls.constance_provider_list_url = reverse(
            'fooitemmodel_constance_provider_signed_request_required-list'
        )
        cls.constance_list_url = reverse(
            'fooitemmodel_constance_signed_request_required-list'
        )

        cls.AUTH_USER = 'test_auth_backend_user'
        cls.AUTH_USER_EMAIL = 'test_ska_auth_user@mail.example.com'
        cls.AUTH_USER_FIRST_NAME = 'John'
        cls.AUTH_USER_LAST_NAME = 'Doe'
        cls.PROVIDER_NAME = 'client_1.admins'

    def setUp(self):
        """Set up."""
        self.foo_items = factories.FooItemFactory.create_batch(10)
        self.foo_item = self.foo_items[0]

        self.provider_detail_url = reverse(
            'fooitemmodel_provider_signed_request_required-detail',
            kwargs={'id': self.foo_item.pk}
        )
        self.detail_url = reverse(
            'fooitemmodel_signed_request_required-detail',
            kwargs={'id': self.foo_item.pk}
        )

        self.constance_provider_detail_url = reverse(
            'fooitemmodel_constance_provider_signed_request_required-detail',
            kwargs={'id': self.foo_item.pk}
        )
        self.constance_detail_url = reverse(
            'fooitemmodel_constance_signed_request_required-detail',
            kwargs={'id': self.foo_item.pk}
        )

        factories.SkaSecretKeyConstanceFactory()
        factories.SkaProvidersConstanceFactory()

    def _test_permissions_request_not_signed_fail(self, url):
        """Fail test permissions request not signed.

        :return:
        """
        data = {}
        response = self.client.get(url, data)
        self.assertIn(
            response.status_code,
            (
                status.HTTP_401_UNAUTHORIZED,
                status.HTTP_403_FORBIDDEN,
            )
        )

    def _test_permissions_request_signed(self,
                                         secret_key,
                                         expected_response_code,
                                         url,
                                         auth_user=None,
                                         auth_user_email=None,
                                         provider_name=None):
        """Test permissions signed requests.

        :return:
        """
        if not auth_user:
            auth_user = self.AUTH_USER
        if not auth_user_email:
            auth_user_email = self.AUTH_USER_EMAIL

        # Testing signed URLs
        extra = {
            'email': auth_user_email,
            'first_name': self.AUTH_USER_FIRST_NAME,
            'last_name': self.AUTH_USER_LAST_NAME,
        }

        if provider_name:
            extra.update({DEFAULT_PROVIDER_PARAM: provider_name})

        signed_list_url_url = sign_url(
            auth_user=auth_user,
            secret_key=secret_key,
            url=url,
            extra=extra
        )

        data = {}
        if not isinstance(expected_response_code, (tuple, list)):
            expected_response_code = [expected_response_code]
        response = self.client.get(signed_list_url_url, data)
        self.assertIn(response.status_code, expected_response_code)


@pytest.mark.django_db
class DRFIntegrationPermissionsTestCase(BaseDRFIntegrationPermissionsTestCase):
    """Django REST framework integration permissions test case."""

    pytestmark = pytest.mark.django_db

    # **************************************************************
    # ********************* Default permissions ********************
    # **************************************************************

    def test_permissions_provider_list_request_not_signed_fail(self):
        """Fail test permissions provider list request not signed.

        :return:
        """
        self._test_permissions_request_not_signed_fail(
            self.provider_list_url
        )

    def test_permissions_provider_detail_request_not_signed_fail(self):
        """Fail test permissions provider detail request not signed.

        :return:
        """
        self._test_permissions_request_not_signed_fail(
            self.provider_detail_url
        )

    def test_permissions_list_request_not_signed_fail(self):
        """Fail test permissions list request not signed.

        :return:
        """
        self._test_permissions_request_not_signed_fail(
            self.list_url
        )

    def test_permissions_detail_request_not_signed_fail(self):
        """Fail test permissions detail request not signed.

        :return:
        """
        self._test_permissions_request_not_signed_fail(
            self.detail_url
        )

    def test_provider_permissions_list_request_signed(self):
        """Test permissions signed provider list request.

        :return:
        """
        secret_key = PROVIDERS[self.PROVIDER_NAME]['SECRET_KEY']
        self._test_permissions_request_signed(
            secret_key,
            status.HTTP_200_OK,
            self.provider_list_url,
            auth_user=self.AUTH_USER,
            auth_user_email=self.AUTH_USER_EMAIL,
            provider_name=self.PROVIDER_NAME
        )

    def test_provider_permissions_detail_request_signed(self):
        """Test permissions signed provider detail request.

        :return:
        """
        secret_key = PROVIDERS[self.PROVIDER_NAME]['SECRET_KEY']
        self._test_permissions_request_signed(
            secret_key,
            status.HTTP_200_OK,
            self.provider_detail_url,
            auth_user=self.AUTH_USER,
            auth_user_email=self.AUTH_USER_EMAIL,
            provider_name=self.PROVIDER_NAME
        )

    def test_provider_permissions_list_request_signed_wrong_secret_key_fail(
            self
    ):
        """Test permissions signed provider list request wrong secret key.

        :return:
        """
        secret_key = PROVIDERS[self.PROVIDER_NAME]['SECRET_KEY']
        self._test_permissions_request_signed(
            "{}w".format(secret_key),
            (
                status.HTTP_401_UNAUTHORIZED,
                status.HTTP_403_FORBIDDEN,
            ),
            self.provider_list_url,
            auth_user=self.AUTH_USER,
            auth_user_email=self.AUTH_USER_EMAIL,
            provider_name=self.PROVIDER_NAME
        )

    def test_provider_permissions_detail_request_signed_wrong_secret_key_fail(
            self
    ):
        """Test permissions signed provider detail request wrong secret key.

        :return:
        """
        secret_key = PROVIDERS[self.PROVIDER_NAME]['SECRET_KEY']
        self._test_permissions_request_signed(
            "{}w".format(secret_key),
            (
                status.HTTP_401_UNAUTHORIZED,
                status.HTTP_403_FORBIDDEN,
            ),
            self.provider_detail_url,
            auth_user=self.AUTH_USER,
            auth_user_email=self.AUTH_USER_EMAIL,
            provider_name=self.PROVIDER_NAME
        )

    def test_permissions_list_request_signed(self):
        """Test permissions signed list request.

        :return:
        """
        secret_key = SECRET_KEY
        self._test_permissions_request_signed(
            secret_key,
            status.HTTP_200_OK,
            self.list_url,
            auth_user=self.AUTH_USER,
            auth_user_email=self.AUTH_USER_EMAIL
        )

    def test_permissions_detail_request_signed(self):
        """Test permissions signed detail request.

        :return:
        """
        secret_key = SECRET_KEY
        self._test_permissions_request_signed(
            secret_key,
            status.HTTP_200_OK,
            self.detail_url,
            auth_user=self.AUTH_USER,
            auth_user_email=self.AUTH_USER_EMAIL
        )

    def test_permissions_list_request_signed_wrong_secret_key_fail(self):
        """Test permissions signed list request wrong secret key.

        :return:
        """
        secret_key = SECRET_KEY
        self._test_permissions_request_signed(
            "{}w".format(secret_key),
            (
                status.HTTP_401_UNAUTHORIZED,
                status.HTTP_403_FORBIDDEN,
            ),
            self.list_url,
            auth_user=self.AUTH_USER,
            auth_user_email=self.AUTH_USER_EMAIL
        )

    def test_permissions_detail_request_signed_wrong_secret_key_fail(self):
        """Test permissions signed detail request wrong secret key.

        :return:
        """
        secret_key = SECRET_KEY
        self._test_permissions_request_signed(
            "{}w".format(secret_key),
            (
                status.HTTP_401_UNAUTHORIZED,
                status.HTTP_403_FORBIDDEN,
            ),
            self.detail_url,
            auth_user=self.AUTH_USER,
            auth_user_email=self.AUTH_USER_EMAIL
        )


@pytest.mark.django_db
class DRFIntegrationPermissionsConstanceTestCase(
    BaseDRFIntegrationPermissionsTestCase
):
    """Django REST framework integration permissions constance test case."""

    pytestmark = pytest.mark.django_db

    # **************************************************************
    # ********************* Default permissions ********************
    # **************************************************************

    @override_settings(**OVERRIDE_SETTINGS_KWARGS)
    def test_permissions_provider_list_request_not_signed_fail(self):
        """Fail test permissions provider list request not signed.

        :return:
        """
        self._test_permissions_request_not_signed_fail(
            self.constance_provider_list_url
        )

    @override_settings(**OVERRIDE_SETTINGS_KWARGS)
    def test_permissions_provider_detail_request_not_signed_fail(self):
        """Fail test permissions provider detail request not signed.

        :return:
        """
        self._test_permissions_request_not_signed_fail(
            self.constance_provider_detail_url
        )

    @override_settings(**OVERRIDE_SETTINGS_KWARGS)
    def test_permissions_list_request_not_signed_fail(self):
        """Fail test permissions list request not signed.

        :return:
        """
        self._test_permissions_request_not_signed_fail(
            self.constance_list_url
        )

    @override_settings(**OVERRIDE_SETTINGS_KWARGS)
    def test_permissions_detail_request_not_signed_fail(self):
        """Fail test permissions detail request not signed.

        :return:
        """
        self._test_permissions_request_not_signed_fail(
            self.constance_detail_url
        )

    @override_settings(**OVERRIDE_SETTINGS_KWARGS)
    def test_provider_permissions_list_request_signed(self):
        """Test permissions signed provider list request.

        :return:
        """
        secret_key = config.SKA_PROVIDERS[self.PROVIDER_NAME]['SECRET_KEY']
        self._test_permissions_request_signed(
            secret_key,
            status.HTTP_200_OK,
            self.constance_provider_list_url,
            auth_user=self.AUTH_USER,
            auth_user_email=self.AUTH_USER_EMAIL,
            provider_name=self.PROVIDER_NAME
        )

    @override_settings(**OVERRIDE_SETTINGS_KWARGS)
    def test_provider_permissions_detail_request_signed(self):
        """Test permissions signed provider detail request.

        :return:
        """
        secret_key = config.SKA_PROVIDERS[self.PROVIDER_NAME]['SECRET_KEY']
        self._test_permissions_request_signed(
            secret_key,
            status.HTTP_200_OK,
            self.constance_provider_detail_url,
            auth_user=self.AUTH_USER,
            auth_user_email=self.AUTH_USER_EMAIL,
            provider_name=self.PROVIDER_NAME
        )

    @override_settings(**OVERRIDE_SETTINGS_KWARGS)
    def test_provider_permissions_list_request_signed_wrong_secret_key_fail(
            self
    ):
        """Test permissions signed provider list request wrong secret key.

        :return:
        """
        secret_key = config.SKA_PROVIDERS[self.PROVIDER_NAME]['SECRET_KEY']
        self._test_permissions_request_signed(
            "{}w".format(secret_key),
            (
                status.HTTP_401_UNAUTHORIZED,
                status.HTTP_403_FORBIDDEN,
            ),
            self.constance_provider_list_url,
            auth_user=self.AUTH_USER,
            auth_user_email=self.AUTH_USER_EMAIL,
            provider_name=self.PROVIDER_NAME
        )

    @override_settings(**OVERRIDE_SETTINGS_KWARGS)
    def test_provider_permissions_detail_request_signed_wrong_secret_key_fail(
            self
    ):
        """Test permissions signed provider detail request wrong secret key.

        :return:
        """
        secret_key = config.SKA_PROVIDERS[self.PROVIDER_NAME]['SECRET_KEY']
        self._test_permissions_request_signed(
            "{}w".format(secret_key),
            (
                status.HTTP_401_UNAUTHORIZED,
                status.HTTP_403_FORBIDDEN,
            ),
            self.constance_provider_detail_url,
            auth_user=self.AUTH_USER,
            auth_user_email=self.AUTH_USER_EMAIL,
            provider_name=self.PROVIDER_NAME
        )

    @override_settings(**OVERRIDE_SETTINGS_KWARGS)
    def test_permissions_list_request_signed(self):
        """Test permissions signed list request.

        :return:
        """
        secret_key = config.SKA_SECRET_KEY
        self._test_permissions_request_signed(
            secret_key,
            status.HTTP_200_OK,
            self.constance_list_url,
            auth_user=self.AUTH_USER,
            auth_user_email=self.AUTH_USER_EMAIL
        )

    @override_settings(**OVERRIDE_SETTINGS_KWARGS)
    def test_permissions_detail_request_signed(self):
        """Test permissions signed detail request.

        :return:
        """
        secret_key = config.SKA_SECRET_KEY
        self._test_permissions_request_signed(
            secret_key,
            status.HTTP_200_OK,
            self.constance_detail_url,
            auth_user=self.AUTH_USER,
            auth_user_email=self.AUTH_USER_EMAIL
        )

    @override_settings(**OVERRIDE_SETTINGS_KWARGS)
    def test_permissions_list_request_signed_wrong_secret_key_fail(self):
        """Test permissions signed list request wrong secret key.

        :return:
        """
        secret_key = config.SKA_SECRET_KEY
        self._test_permissions_request_signed(
            "{}w".format(secret_key),
            (
                status.HTTP_401_UNAUTHORIZED,
                status.HTTP_403_FORBIDDEN,
            ),
            self.constance_list_url,
            auth_user=self.AUTH_USER,
            auth_user_email=self.AUTH_USER_EMAIL
        )

    @override_settings(**OVERRIDE_SETTINGS_KWARGS)
    def test_permissions_detail_request_signed_wrong_secret_key_fail(self):
        """Test permissions signed detail request wrong secret key.

        :return:
        """
        secret_key = config.SKA_SECRET_KEY
        self._test_permissions_request_signed(
            "{}w".format(secret_key),
            (
                status.HTTP_401_UNAUTHORIZED,
                status.HTTP_403_FORBIDDEN,
            ),
            self.constance_detail_url,
            auth_user=self.AUTH_USER,
            auth_user_email=self.AUTH_USER_EMAIL
        )


if __name__ == "__main__":
    # Tests
    unittest.main()



from __future__ import absolute_import
import logging

from ....... import validate_signed_request_data
from .......defaults import (
    DEFAULT_SIGNATURE_PARAM,
    DEFAULT_AUTH_USER_PARAM,
    DEFAULT_VALID_UNTIL_PARAM,
    DEFAULT_EXTRA_PARAM,
)
from .......exceptions import ImproperlyConfigured, InvalidData

from ....utils import get_provider_data

from rest_framework import permissions

__title__ = 'ska.contrib.django.ska.integration.drf.permissions.base'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2019 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'AbstractSignedRequestRequired',
    'BaseSignedRequestRequired',
    'BaseProviderSignedRequestRequired',
)

logger = logging.getLogger(__file__)


class AbstractSignedRequestRequired(permissions.BasePermission):
    """Signed request required permission."""

    def get_settings(self):
        """Get settings.

        :return:
        """
        raise NotImplementedError(
            "You should implement this method in your permission class"
        )

    def get_secret_key(self, request_data, request=None, view=None, obj=None):
        """Get secret key.

        :param request_data:
        :param request:
        :param view:
        :param obj:
        :return:
        """
        raise NotImplementedError(
            "You should implement this method in your permission class"
        )

    def validate_signed_request(self, request, view, obj=None):
        """Validate signed request.

        :param request:
        :param view:
        :param obj:
        :return:
        """
        request_data = request.GET.dict()

        secret_key = self.get_secret_key(
            request_data,
            request,
            view,
            obj
        )

        if not secret_key:
            return False

        try:
            # If authentication/data validation failed.
            validation_result = validate_signed_request_data(
                data=request_data,
                secret_key=secret_key,
                signature_param=DEFAULT_SIGNATURE_PARAM,
                auth_user_param=DEFAULT_AUTH_USER_PARAM,
                valid_until_param=DEFAULT_VALID_UNTIL_PARAM,
                extra_param=DEFAULT_EXTRA_PARAM
            )
            return validation_result.result
        except (ImproperlyConfigured, InvalidData) as err:
            logger.debug(str(err))
            return False

    def has_permission(self, request, view):
        return self.validate_signed_request(request, view)

    def has_object_permission(self, request, view, obj):
        return self.validate_signed_request(request, view, obj)


class BaseSignedRequestRequired(AbstractSignedRequestRequired):
    """Signed request required permission."""

    def get_secret_key(self, request_data, request=None, view=None, obj=None):
        """Get secret key.

        :param request_data:
        :param request:
        :param view:
        :param obj:
        :return:
        """
        settings = self.get_settings()
        return settings.get('SECRET_KEY', None)


class BaseProviderSignedRequestRequired(AbstractSignedRequestRequired):
    """Provider signed request required permission."""

    def get_secret_key(self, request_data, request=None, view=None, obj=None):
        """Get secret key.

        :param request_data:
        :param request:
        :param view:
        :param obj:
        :return:
        """
        provider_settings = self.get_settings()
        provider_data = get_provider_data(request_data, provider_settings)
        if provider_data:
            secret_key = provider_data['SECRET_KEY']
            return secret_key



from .base import SignatureValidationResult, AbstractSignature
from .defaults import (
    SIGNATURE_LIFETIME,
    TIMESTAMP_FORMAT,
    DEFAULT_URL_SUFFIX,
    DEFAULT_SIGNATURE_PARAM,
    DEFAULT_AUTH_USER_PARAM,
    DEFAULT_VALID_UNTIL_PARAM,
    DEFAULT_EXTRA_PARAM,
)
from .signatures import (
    Signature,
    HMACMD5Signature,
    HMACSHA1Signature,
    HMACSHA224Signature,
    HMACSHA256Signature,
    HMACSHA384Signature,
    HMACSHA512Signature,
)
from .shortcuts import (
    sign_url,
    signature_to_dict,
    validate_signed_request_data,
    extract_signed_request_data,
)
from .utils import RequestHelper

__title__ = 'ska'
__version__ = '1.7.5'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2019 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'sign_url',
    'signature_to_dict',
    'validate_signed_request_data',
    'extract_signed_request_data',
    'Signature',
    'RequestHelper',
    'SignatureValidationResult',
    'HMACMD5Signature',
    'HMACSHA224Signature',
    'HMACSHA256Signature',
    'HMACSHA384Signature',
    'HMACSHA512Signature',
)



from __future__ import absolute_import, print_function

import logging
import unittest

from django.test import Client, TransactionTestCase

import pytest

import factories

from .helpers import (
    generate_data,
    log_info,
    NUM_ITEMS,
)

__title__ = 'ska.contrib.django.ska.tests.test_decorators'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2019 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'SkaDecoratorsTest',
)

logger = logging.getLogger(__name__)


# *********************************************************************
# *********************************************************************
# *********************************************************************

SKA_TEST_USER_USERNAME = factories.TEST_ADMIN_USERNAME
SKA_TEST_USER_PASSWORD = factories.TEST_PASSWORD


@pytest.mark.django_db
class SkaDecoratorsTest(TransactionTestCase):
    """Testing model- and view- decorators."""

    pytestmark = pytest.mark.django_db

    def setUp(self):
        items = generate_data(NUM_ITEMS)

        # Testing the URLs
        self.item = items[0]

    @log_info
    def test_01_model_decorator(self):
        """Test the ``ska.contrib.django.ska.decorators.sign_url``."""
        # Testing signed URLs
        signed_absolute_url = self.item.get_signed_absolute_url()
        self.assertIsNotNone(signed_absolute_url)
        return signed_absolute_url

    @log_info
    def test_02_view_decorator_with_signed_url(self):
        """Test view decorator with signed URL.

        Test ``ska.contrib.django.ska.decorators.validate_signed_request``.
        """
        flow = []

        # Testing signed URLs
        signed_absolute_url = self.item.get_signed_absolute_url()
        self.assertIsNotNone(signed_absolute_url)
        flow.append(('Signed absolute URL', signed_absolute_url))

        # Testing view with signed URL
        client = Client()
        response = client.get(signed_absolute_url, {})
        response_status_code = getattr(response, 'status_code', None)
        self.assertIn(response_status_code, (200, 201, 202))
        flow.append(
            ('Response status code for signed URL', response_status_code)
        )

        return flow

    @log_info
    def test_03_view_decorator_with_unsigned_url(self):
        """Test view decorator with unsigned URL.

        Test the
        ``ska.contrib.django.ska.decorators.validate_signed_request`` view
        decorator with unsigned URL.
        """
        flow = []

        # Testing unsigned URLs
        absolute_url = self.item.get_absolute_url()
        self.assertTrue(absolute_url is not None)
        flow.append(('Unsigned absolute URL', absolute_url))

        # Testing view with signed URL
        client = Client()
        response = client.get(absolute_url, {})
        response_status_code = getattr(response, 'status_code', None)
        response_content = getattr(response, 'content', "")
        self.assertIn(response_status_code, (401,))
        flow.append(
            ('Response status code for unsigned URL', response_status_code)
        )
        flow.append(
            ('Response content for unsigned URL', response_content)
        )

        return flow

    @log_info
    def test_04_class_based_view_decorator_with_signed_url(self):
        """Test class based view decorator with signed URL.

        Test ``ska.contrib.django.ska.decorators.validate_signed_request``.
        """
        flow = []

        # Testing signed URLs
        signed_absolute_url = self.item \
            .get_signed_class_based_absolute_url()
        self.assertIsNotNone(signed_absolute_url)
        flow.append(('Signed absolute URL', signed_absolute_url))

        # Testing view with signed URL
        client = Client()
        response = client.get(signed_absolute_url, {})
        response_status_code = getattr(response, 'status_code', None)
        self.assertIn(response_status_code, (200, 201, 202))
        flow.append(
            ('Response status code for signed URL', response_status_code)
        )

        return flow

    @log_info
    def test_05_class_based_view_decorator_with_unsigned_url(self):
        """Test class based view decorator with unsigned URL.

        Test the
        ``ska.contrib.django.ska.decorators.validate_signed_request`` view
        decorator with unsigned URL.
        """
        flow = []

        # Testing unsigned URLs
        absolute_url = self.item.get_cbv_absolute_url()
        self.assertTrue(absolute_url is not None)
        flow.append(('Unsigned absolute URL', absolute_url))

        # Testing view with signed URL
        client = Client()
        response = client.get(absolute_url, {})
        response_status_code = getattr(response, 'status_code', None)
        response_content = getattr(response, 'content', "")
        self.assertIn(response_status_code, (401,))
        flow.append(
            ('Response status code for unsigned URL', response_status_code)
        )
        flow.append(
            ('Response content for unsigned URL', response_content)
        )

        return flow


if __name__ == "__main__":
    # Tests
    unittest.main()
